use leptos::{
    Params,
    prelude::*
};
use serde::{Deserialize, Serialize};
use leptos_router::{
    components::Outlet, hooks::use_params, params::Params
};

use crate::{i18n::*, services::directus::Asset};


#[derive(Clone, Debug, Serialize, Deserialize, Default)]
pub struct BlogPostTranslations {
    title: String,
    content: String
}

#[derive(Clone, Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct BlogPost {
    slug: String,
    translations: Option<BlogPostTranslations>,
    illustration: Asset,
}

#[server(GetBlog, "/api", "GetJson")]
pub async fn get_blog_posts(locale: String) -> Result<Vec<BlogPost>, ServerFnError> {
    crate::services::directus::get_client()
        .get_all_items_in_collection::<BlogPost>(&locale, "blog_posts").await
        .map_err(|e| ServerFnError::ServerError(e.to_string()))
}

#[component]
pub fn Blog() -> impl IntoView {
    let i18n = use_i18n();

    let blog_posts = Resource::new(move || i18n.get_locale(), move |locale| get_blog_posts(locale.to_string()));

    view! {
        <Outlet/>
        <main class="main-width blog-wrapper">
            <h2>{t!(i18n, blog_latest_posts)}:</h2>
            <Suspense fallback=move || view! { <div>"Loading..."</div> }>
                {move || blog_posts.get()
                    .and_then(|blog_posts| blog_posts.ok())
                    .and_then(|blog_posts| {
                        view! {
                            <div class="blog-entries">
                                {
                                    blog_posts.into_iter().map(|post| {
                                        view! {
                                            <a class="blog-thumbnail" href={format!("/blog/{}", post.slug)}>
                                                <img src=format!("https://directus.gerome.fi/assets/{}?width=600&height=400&fit=cover&format=auto&quality=80&withoutEnlargement=true", post.illustration.id)/>
                                                <p class="blog-title">{post.translations.unwrap_or_default().title}</p>
                                            </a>
                                        }
                                    }).collect::<Vec<_>>()
                                }
                            </div>
                        }.into()
                    })
                 }
            </Suspense>
        </main>
    }
}

#[derive(Params, PartialEq)]
struct BlogPostParams {
    slug: Option<String>,
}

#[component]
pub fn BlogPost() -> impl IntoView {
    let i18n = use_i18n();
    let params = use_params::<BlogPostParams>();
    let slug = move || {
        params .read()
            .as_ref()
            .ok()
            .and_then(|params| params.slug.clone())
            .unwrap_or_default()
    };

    let blog_posts = Resource::new(move || i18n.get_locale(), move |locale| get_blog_posts(locale.to_string()));

    view! {
        <Suspense fallback=move || view! { <main class="main-width blog-entry-wrapper"><div>"Loading..."</div></main> }>
            {move || blog_posts.get()
                .and_then(|blog_posts| blog_posts.ok())
                .and_then(|blog_posts| blog_posts.iter()
                    .find(|post| post.slug == slug())
                    .map(|post| post.clone()))
                .and_then(|post| {
                    let translations = post.translations.unwrap_or_default();
                    let html = markdown::to_html(&translations.content);
                    view! {
                        <div class="blog-open-entry">
                            <div class="blog-illustration">
                                <img src=format!("https://directus.gerome.fi/assets/{}?width=1600&height=1000&fit=inside&format=auto&quality=90&withoutEnlargement=true", post.illustration.id)/>
                                <h2 class="blog-title"><p class="main-width">{translations.title}</p></h2>
                            </div>
                            <main class="main-width blog-entry-wrapper">
                                <div class="markdown" inner_html=html/>
                            </main>
                        </div>
                     }.into()
                 })
             }
        </Suspense>
    }
}
