use leptos::{
    Params,
    prelude::*
};
use serde::{Deserialize, Serialize};
use leptos_router::{
    components::Outlet, hooks::use_params, params::Params
};

use crate::{i18n::*, services::directus::Asset};

#[derive(Clone, Debug, Serialize, Deserialize, Default)]
pub struct ImageTranslations {
    subtitle: Option<String>,
    description: String
}

#[derive(Clone, Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Image {
    slug: String,
    title: String,
    translations: Option<ImageTranslations>,
    watermarked: Asset,
    original: Asset,
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Gallery {
    images: Vec<Image>
}

#[server(GetGallery, "/api", "GetJson")]
pub async fn get_gallery(locale: String) -> Result<Gallery, ServerFnError> {
    let all_images = crate::services::directus::get_client().get_all_items_in_collection::<Image>(&locale, "image").await.unwrap_or(vec![]);
    crate::services::directus::get_client().get_item_json("gallery", "gallery").await
        .map_err(|e| ServerFnError::ServerError(e.to_string()))
        .and_then(|gallery_json| {
            let images_vec_json = gallery_json["data"]["images"].as_array().unwrap();
            let images = images_vec_json.into_iter().map(|image_json| {
                all_images.iter().find(|image| image.slug == image_json["item"])
                    .map(|found_image| found_image.clone())
            });
            let collected_images: Option<Vec<_>> = images.collect();
            match collected_images {
                Some(images) => Ok(Gallery{ images }),
                _ => Err(ServerFnError::ServerError("aaa".to_string()))
            }
        })
}

#[component]
pub fn Gallery() -> impl IntoView {
    let i18n = use_i18n();

    let gallery = Resource::new(move || i18n.get_locale(), move |locale| get_gallery(locale.to_string()));

    view! {
        <Outlet/>
        <main class="main-width gallery-wrapper">
            <h2>{t!(i18n, gallery)}</h2>
            <Suspense fallback=move || view! { <div>"Loading..."</div> }>
                {move || gallery.get()
                    .and_then(|gallery| gallery.ok())
                    .and_then(|gallery| {
                        view! {
                            <div class="gallery-entries">
                                {
                                    gallery.images.into_iter().map(|image| {
                                        view! {
                                            <div class="gallery-thumbnail" >
                                                <a href={format!("/gallery/{}", image.slug)}>
                                                    <img src=format!("https://directus.gerome.fi/assets/{}?width=400&height=400&fit=inside&format=auto&quality=80&withoutEnlargement=true", image.original.id)/>
                                                    <p>{image.title}</p>
                                                </a>
                                            </div>
                                        }
                                    }).collect::<Vec<_>>()
                                }
                            </div>
                        }.into()
                    })
                 }
            </Suspense>
        </main>
    }
}

#[derive(Params, PartialEq)]
struct GalleryEntryParams {
    slug: Option<String>,
}

#[component]
pub fn GalleryEntry() -> impl IntoView {
    let i18n = use_i18n();
    let params = use_params::<GalleryEntryParams>();
    let slug = move || {
        params .read()
            .as_ref()
            .ok()
            .and_then(|params| params.slug.clone())
            .unwrap_or_default()
    };

    let gallery = Resource::new(move || i18n.get_locale(), move |locale| get_gallery(locale.to_string()));

    view! {
        <Suspense fallback=move || view! { <div>"Loading..."</div> }>
            {move || gallery.get()
                .and_then(|gallery| gallery.ok())
                .and_then(|gallery| gallery.images.iter()
                    .find(|image| image.slug == slug())
                    .map(|image| image.clone()))
                .and_then(|image| {
                    let translations = image.translations.unwrap_or_default();
                    let subtitle = translations.subtitle.unwrap_or_default();
                    view! {
                        <div class="main-width gallery-open-entry">
                            <h2 class="gallery-title">{image.title}</h2>
                            <img src=format!("https://directus.gerome.fi/assets/{}?width=1600&height=1600&fit=inside&format=auto&quality=90&withoutEnlargement=true", image.watermarked.id)/>
                            {if subtitle.len() > 0 {
                                Some(view!{<h4 class="gallery-subtitle">{subtitle}</h4>})
                            } else {
                                None
                            }}
                            <p class="gallery-description">{translations.description}</p>
                        </div>
                     }.into()
                 })
             }
        </Suspense>
    }
}
