use std::collections::HashMap;
use std::env;
use leptos::prelude::*;
use serde::{Deserialize, Serialize};

pub struct DirectusClient {
    access_token: String,
    base_url: String,
    reqwest_client: reqwest::Client
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Asset {
    pub id: String
}

impl DirectusClient {
    pub fn new(access_token: &str, base_url: &str) -> DirectusClient {
        let reqwest_client = reqwest::Client::new();
        DirectusClient {
            access_token: access_token.to_string(),
            base_url: base_url.to_string(),
            reqwest_client
        }
    }

    pub async fn list_items_in_collection(&self, collection: &str, primary_key: &str) -> Result<Vec<String>> {
        let response = self.reqwest_client.get(format!("{}/items/{}?fields={}&filter[status][_eq]=published", self.base_url, collection, primary_key))
            .header("Authorization", format!("Bearer {}", self.access_token))
            .send().await?;
        let items_json = response.json::<Vec<HashMap<String, String>>>().await?;
        let ids = items_json.iter().map(|item| item[&primary_key.to_string()].clone()).collect();
        Ok(ids)
    }

    pub async fn get_item_json(&self, collection: &str, id: &str) -> Result<serde_json::Value> {
        let response = self.reqwest_client.get(format!("{}/items/{}/{}?fields=*,*.*,translations.*&filter[status][_eq]=published", self.base_url, collection, id))
            .header("Authorization", format!("Bearer {}", self.access_token))
            .send().await?;
        let json = &response.json::<serde_json::Value>().await?;
        Ok(json.clone())
    }

    pub async fn get_item<T: for<'a> Deserialize<'a>>(&self, locale: &str, collection: &str, id: &str) -> Result<T> {
        let response = self.reqwest_client.get(format!("{}/items/{}/{}?fields=*,*.*,translations.*&filter[status][_eq]=published", self.base_url, collection, id))
            .header("Authorization", format!("Bearer {}", self.access_token))
            .send().await?;
        let json = &response.json::<serde_json::Value>().await?;
        let translations_json = json["data"]["translations"].clone();
        let found_translation = match translations_json.as_array() {
            Some(translations) => translations.iter().find(|translation| translation["languages_code"] == locale),
            _ => None
        };
        let mut new_json = json["data"].clone();
        new_json["translations"] = found_translation.unwrap_or(&serde_json::Value::Null).clone();
        let item = serde_json::from_value::<T>(new_json)?;
        Ok(item)
    }

    pub async fn get_all_items_in_collection<T: for<'a> Deserialize<'a>>(&self, locale: &str, collection: &str) -> Result<Vec<T>> {
        let response = self.reqwest_client.get(format!("{}/items/{}?fields=*,*.*,translations.*&sort=-date_created&filter[status][_eq]=published", self.base_url, collection))
            .header("Authorization", format!("Bearer {}", self.access_token))
            .send().await?;
        let json = &response.json::<serde_json::Value>().await?;
        let items_json = json["data"].as_array().unwrap_or(&vec!()).clone();
        let items = items_json.iter().map(|item_json| {
            let translations_json = item_json["translations"].clone();
            let found_translation = match translations_json.as_array() {
                Some(translations) => translations.iter().find(|translation| translation["languages_code"] == locale),
                _ => None
            };
            let mut new_json = item_json.clone();
            new_json["translations"] = found_translation.unwrap_or(&serde_json::Value::Null).clone();
            let item = serde_json::from_value::<T>(new_json);
            item.unwrap()
        }).collect();
        Ok(items)
    }
}

pub fn get_client() -> DirectusClient {
    let access_token = env::var("DIRECTUS_ACCESS_TOKEN").unwrap();
    DirectusClient::new(&access_token, "https://directus.gerome.fi")
}
